@@ -19,8 +19,9 @@ from falocalrepo_database.tables import UsersColumns
 from falocalrepo_database.tables import journals_table
 from falocalrepo_database.tables import submissions_table
 from falocalrepo_database.tables import users_table
+from falocalrepo_database.util import compare_version
 
-default_sort: dict[str, str] = {submissions_table: "id", journals_table: "id", users_table: "username"}
+default_sort: dict[str, str] = {submissions_table: "date", journals_table: "date", users_table: "username"}
 default_order: dict[str, str] = {submissions_table: "desc", journals_table: "desc", users_table: "asc"}
 
 
@@ -74,9 +75,11 @@ def query_to_sql(query: str, default_field: str, likes: list[str] = None, aliase
 class Database(_Database):
     def __init__(self, database_path: Path):
         _Database.check_connection(database_path)
-        super().__init__(database_path, read_only=not access(database_path, W_OK))
+        super().__init__(database_path, read_only=not access(database_path, W_OK), check_version=False)
         if not self.is_formatted:
             raise DatabaseError("Database not formatted")
+        elif err := compare_version(self.version, patch=False):
+            raise err
 
     def __enter__(self):
         return self
@@ -123,10 +126,18 @@ class Database(_Database):
                                       ) -> tuple[Path | None, Path | None]:
         return self.submissions.get_submission_files(submission_id)
 
+    @cache
+    def _load_submission_comments_cached(self, submission_id: int, *, _cache=None) -> list[dict]:
+        return self.comments.get_comments_tree(submissions_table, submission_id)
+
     @cache
     def _load_journal_cached(self, journal_id: int, *, _cache=None) -> dict | None:
         return self.journals[journal_id]
 
+    @cache
+    def _load_journal_comments_cached(self, journal_id: int, *, _cache=None) -> list[dict]:
+        return self.comments.get_comments_tree(journals_table, journal_id)
+
     @cache
     def _load_prev_next_cached(self, table: str, item_id: int | str, *, _cache=None) -> tuple[int, int]:
         db_table: Table = self[table]
@@ -165,11 +176,12 @@ class Database(_Database):
         order = order or default_order[table]
         db_table: Table
 
-        if table in (submissions_table, journals_table):
+        if (table := table.upper()) in (submissions_table.upper(), journals_table.upper()):
             cols_results = [SubmissionsColumns.ID.value, SubmissionsColumns.AUTHOR.value,
                             SubmissionsColumns.DATE.value, SubmissionsColumns.TITLE.value]
             db_table = self.submissions if table == submissions_table else self.journals
         else:
+            default_field = "username"
             cols_results = [UsersColumns.USERNAME.value, UsersColumns.FOLDERS.value, UsersColumns.ACTIVE.value]
             db_table = self.users
 
@@ -188,10 +200,10 @@ class Database(_Database):
                                     "message": "description",
                                     "filename": "fileurl",
                                     "any": f"({'||'.join(cols_table)})"},
-                                   score=sort == "relevance")
+                                   score=sort.lower() == "relevance")
 
         results: list[dict]
-        if sort == "relevance":
+        if sort.lower() == "relevance":
             results = [
                 dict(zip([c.name for c in cols_results] + ["RELEVANCE"], s)) for s in
                 db_table.select_sql(f"RELEVANCE > 0", values,
@@ -240,14 +252,20 @@ class Database(_Database):
     def load_submission_files(self, submission_id: int) -> tuple[Path | None, Path | None]:
         return self._load_submission_files_cached(submission_id, _cache=self.m_time)
 
+    def load_submission_comments(self, submission_id: int) -> list[dict]:
+        return self._load_submission_comments_cached(submission_id, _cache=self.m_time)
+
     def load_journal(self, journal_id: int) -> dict | None:
         return self._load_journal_cached(journal_id, _cache=self.m_time)
 
+    def load_journal_comments(self, journal_id: int) -> list[dict]:
+        return self._load_journal_comments_cached(journal_id, _cache=self.m_time)
+
     def load_prev_next(self, table: str, item_id: int | str) -> tuple[int, int]:
         return self._load_prev_next_cached(table, item_id, _cache=self.m_time)
 
     def load_search(self, table: str, query: str, sort: str, order: str):
-        return self._load_search_cached(table, query, sort, order, _cache=self.m_time)
+        return self._load_search_cached(table.lower(), query.lower(), sort.lower(), order.lower(), _cache=self.m_time)
 
     def load_files_folder(self) -> Path:
         return self._load_files_folder_cached(_cache=self.m_time)
@@ -264,12 +282,18 @@ class Database(_Database):
     def load_submission_uncached(self, submission_id: int) -> dict | None:
         return self._load_submission_cached.__wrapped__(self, submission_id)
 
+    def load_submission_comments_uncached(self, submission_id: int) -> list[dict]:
+        return self._load_submission_comments_cached.__wrapped__(self, submission_id)
+
     def load_submission_files_uncached(self, submission_id: int) -> tuple[Path | None, Path | None]:
         return self._load_submission_files_cached.__wrapped__(self, submission_id)
 
     def load_journal_uncached(self, journal_id: int) -> dict | None:
         return self._load_journal_cached.__wrapped__(self, journal_id)
 
+    def load_journal_comments_uncached(self, journal_id: int) -> list[dict]:
+        return self._load_journal_comments_cached.__wrapped__(self, journal_id)
+
     def load_prev_next_uncached(self, table: str, item_id: int | str) -> tuple[int, int]:
         return self._load_prev_next_cached.__wrapped__(self, table, item_id)
 
