@@ -126,6 +126,22 @@ bbcode_expressions: list[tuple[Pattern, str]] = [
 app.mount("/static", StaticFiles(directory=settings.static_folder), "static")
 
 
+def flatten_comments(comments: list[dict]) -> list[dict]:
+    return [*{c["ID"]: c for c in [r for c in comments for r in [c, *flatten_comments(c["REPLIES"])]]}.values()]
+
+
+def comments_depth(comments: list[dict], depth: int = 0, root_comment: int = None) -> list[dict]:
+    return flatten_comments([com | {"DEPTH": depth,
+                                    "REPLIES": comments_depth(com.get("REPLIES", []), depth + 1,
+                                                              root_comment or com["ID"]),
+                                    "ROOT": root_comment}
+                             for com in comments])
+
+
+def prepare_comments(comments: list[dict]) -> list[dict]:
+    return [c | {"TEXT": clean_html(c["TEXT"])} for c in comments_depth(comments, 0)]
+
+
 def bbcode_to_html(text: str) -> str:
     return reduce(lambda t, es: es[0].sub(es[1], t), bbcode_expressions, text.strip() + "\n\n")
 
@@ -143,8 +159,17 @@ def clean_html(html: str) -> str:
     return str(html_parsed)
 
 
-def serialise_entry(entry: dict) -> dict:
-    return {k: sorted(v) if isinstance(v, set) else str(v) if isinstance(v, datetime) else v for k, v in entry.items()}
+def serialise_entry(entry: Any) -> Any:
+    if isinstance(entry, dict):
+        return {k: serialise_entry(v) for k, v in entry.items()}
+    elif isinstance(entry, (list, tuple)):
+        return list(map(serialise_entry, entry))
+    elif isinstance(entry, set):
+        return sorted(map(serialise_entry, entry))
+    elif isinstance(entry, datetime):
+        return str(entry)
+    else:
+        return entry
 
 
 def error_response(request: Request, code: int, message: str = None, buttons: list[tuple[str, str]] = None) -> Response:
@@ -186,11 +211,14 @@ def log_settings():
     logger.info(f"Using SSL private key: {settings.ssl_key}") if settings.ssl_key else None
     logger.info(f"Using HTTP Basic authentication") if settings.username or settings.password else None
     if settings.precache:
-        for table in (settings.database.users, settings.database.submissions, settings.database.journals):
-            for order in ("asc", "desc"):
-                logger.info(
-                    f"Caching {table.name.upper()}:{(sort := default_sort[table.name]).upper()}:{order.upper()}")
-                settings.database.load_search(table.name, "", sort, order)
+        for table, order in [
+            (t, o)
+            for t in (settings.database.users, settings.database.submissions, settings.database.journals)
+            for o in ("asc", "desc")
+        ]:
+            logger.info(
+                f"Caching {table.name.upper()}:{(sort := search_settings.sort[table.name]).upper()}:{order.upper()}")
+            settings.database.load_search(table.name, "", "id" if sort.lower() == "date" else sort, order)
 
 
 @app.on_event("shutdown")
@@ -405,10 +433,10 @@ async def serve_search(request: Request, table: str, title: str = None, args: di
     columns_list: list[str]
     column_id: str
 
-    results, columns_table, columns_results, columns_list, column_id, sort, order = settings.database.load_search(
+    results, columns_table, columns_results, columns_list, column_id, _, order = settings.database.load_search(
         table,
         query.lower().strip(),
-        sort,
+        "id" if sort == "date" else sort,
         order
     )
 
@@ -449,9 +477,10 @@ async def serve_submission(request: Request, id_: int):
         "app": app.title,
         "title": f"{sub['TITLE']} by {sub['AUTHOR']}",
         "submission": sub | {"DESCRIPTION": clean_html(sub["DESCRIPTION"])},
-        "file_text": bbcode_to_html(f.read_text(encoding=detect_encoding(f.read_bytes())["encoding"])) if f else None,
+        "file_text": bbcode_to_html(f.read_text(detect_encoding(f.read_bytes())["encoding"], "ignore")) if f else None,
         "filename": f"submission{('.' + sub['FILEEXT']) * bool(sub['FILEEXT'])}",
         "filename_id": f"{sub['ID']:010d}{('.' + sub['FILEEXT']) * bool(sub['FILEEXT'])}",
+        "comments": prepare_comments(settings.database.load_submission_comments(id_)),
         "prev": p,
         "next": n,
         "request": request}),
@@ -509,6 +538,8 @@ async def serve_submission_zip(id_: int, _filename=None):
         z.writestr("description.html", sub["DESCRIPTION"].encode())
         z.writestr("metadata.json", dumps({k: v for k, v in serialise_entry(sub).items()
                                            if k != "DESCRIPTION"}).encode())
+        z.writestr("comments.json", dumps([serialise_entry(c)
+                                           for c in settings.database.load_submission_comments(id_)]).encode())
 
     f_obj.seek(0)
     return StreamingResponse(f_obj, media_type="application/zip")
@@ -525,6 +556,7 @@ async def serve_journal(request: Request, id_: int):
         "app": app.title,
         "title": f"{jrnl['TITLE']} by {jrnl['AUTHOR']}",
         "journal": jrnl | {"CONTENT": clean_html(jrnl["CONTENT"])},
+        "comments": prepare_comments(settings.database.load_journal_comments(id_)),
         "prev": p,
         "next": n,
         "request": request}),
@@ -541,6 +573,8 @@ async def serve_journal_zip(id_: int, _filename=None):
         z.writestr("content.html", jrnl["CONTENT"].encode())
         z.writestr("metadata.json", dumps({k: v for k, v in serialise_entry(jrnl).items()
                                            if k != "CONTENT"}).encode())
+        z.writestr("comments.json", dumps([serialise_entry(c)
+                                           for c in settings.database.load_journal_comments(id_)]).encode())
 
     f_obj.seek(0)
     return StreamingResponse(f_obj, media_type="application/zip")
@@ -584,7 +618,7 @@ async def serve_submission_json(id_: int):
     if not (s := settings.database.load_submission_uncached(id_)):
         raise HTTPException(404)
     else:
-        return s
+        return s | {"COMMENTS": settings.database.load_submission_comments_uncached(id_)}
 
 
 @app.get("/json/journal/{id_}/", response_class=JSONResponse)
@@ -593,7 +627,7 @@ async def serve_journal_json(id_: int):
     if not (j := settings.database.load_journal_uncached(id_)):
         raise HTTPException(404)
     else:
-        return j
+        return j | {"COMMENTS": settings.database.load_journal_comments_uncached(id_)}
 
 
 @app.get("/json/user/{username}/", response_class=JSONResponse)
